
try {
    var Weak = require('./build/Release/weak.node');
} catch (e) {
    try {
        var Weak = require('./build/Debug/weak.node');
    } catch (e) {
        console.error('module hi-lib/weak error:');
        if (e.message.indexOf('different Node.js version using') >= 0) {
            console.error('  was compiled against a different Node.js version using');
        } else {
            console.error(e);
        }
        process.exit();
    }
}

const _weak_proxy_handle = {

    get(weakRef, prop) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return enitiy[prop];
        }
        if (prop === Symbol.iterator) {
            return [][Symbol.iterator];
        }
    },

    set(weakRef, prop, value) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return enitiy[prop] = value;
        }
        return;
    },

    has(weakRef, prop) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return enitiy.hasOwnPropety(prop);
        }
        return false;
    },

    ownKeys(weakRef) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return Object.keys(enitiy);
        }
        return [];
    },

    getPrototypeOf(weakRef) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return Object.getPrototypeOf(enitiy);
        }
        return null;
    },

    setPrototypeOf(weakRef, prototype) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return Object.setPrototypeOf(enitiy, prototype);
        }
        return false;
    },

    isExtensible(weakRef) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return Object.isExtensible(enitiy);
        }
        return false;
    },
    preventExtensions(weakRef) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return Object.preventExtensions(enitiy);
        }
        return false;
    },

    getOwnPropertyDescriptor(weakRef, prop) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return Object.getOwnPropertyDescriptor(enitiy, prop);
        }
    },

    defineProperty(weakRef, property, descriptor) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return Object.defineProperty(enitiy, property, descriptor);
        }
        return false;
    },

    deleteProperty(weakRef, property) {
        let enitiy = weakRef.raw();
        if (enitiy) {
            return Object.deleteProperty(enitiy, property);
        }
        return false;
    }
};

const _weak_dict_handle = {

    get(dict, name) {
        let value = dict[name];
        if (value && typeof value === 'object') {
            if (value.constructor === Weak.WeakRef) {
                return value.raw();
            }
        }
        return value;
    },

    set(dict, name, value) {
        return dict[name] = value && typeof value === 'object' ? exports.create(value) : value;
    }

};

const _gc_listeners = [];

exports.create = function (obj, listener) {
    if (obj.constructor === Weak.WeakRef) {
        return obj;
    }
    return new Weak.WeakRef(obj, listener);
};

exports.wrap = function (obj, listener) {
    return new Proxy(exports.create(obj, listener), _weak_proxy_handle);
};

exports.weakDict = function () {
    return new Proxy({}, _weak_dict_handle);
};

exports.runGC = function (callback) {
    if (typeof callback === 'function') {
        exports.addGCListener(function () {
            callback();
            exports.removeGCListener(callback);
        });
    }
    if (global.gc) {
        global.gc();
    } else {
        Weak.runGC();
    }
};

exports.addGCListener = function (listener) {
    if (_gc_listeners.includes(listener)) {
        return;
    }
    _gc_listeners.push(listener);
    if (_gc_listeners.length === 1) {
        Weak.setGCListener(function () {
            for (let fn of _gc_listeners) {
                fn();
            }
        });
    }
};

exports.removeGCListener = function (listener) {
    let i = _gc_listeners.indexOf(listener);
    if (i >= 0) {
        _gc_listeners.splice(i, 1);
        if (_gc_listeners.length === 0) {
            Weak.setGCListener(undefined);
        }
    }
}

