
#include "./addon.h"

namespace hi_nodejs
{

    using namespace v8;
    using namespace std;

    WeakRefWrap::WeakRefWrap(Local<Object> &weakref, const Nan::FunctionCallbackInfo<Value> &info)
    {
        this->weakref.Reset(weakref);
        this->target.Reset(info[0].As<Object>());
        this->target.SetWeak(this, OnGCCallback, Nan::WeakCallbackType::kParameter);
        if (info[1]->IsFunction())
        {
            this->callback.Reset(info[1].As<Function>());
        }
    }

    WeakRefWrap::~WeakRefWrap()
    {
        this->weakref.Reset();
        this->target.Reset();
        this->callback.Reset();
    }

    void WeakRefWrap::OnGCCallback(const Nan::WeakCallbackInfo<WeakRefWrap> &info)
    {
        Nan::HandleScope scope;
        WeakRefWrap *wrap = info.GetParameter();

        CGCallback(wrap->callback);

        // 删除引用
        Local<Object> weakref = Nan::New<Object>(wrap->weakref);
        Nan::SetInternalFieldPointer(weakref, 0, NULL);
        delete wrap;
    }

    ////////////////////////////////////////////////////////////////////////

    Nan::Persistent<Function> WeakRef::Template;

    WeakRefWrap *WeakRef::UnWrap(Local<Object> weakref)
    {
        assert(weakref->InternalFieldCount() == 1);
        return reinterpret_cast<WeakRefWrap *>(Nan::GetInternalFieldPointer(weakref, 0));
    }

    void WeakRef::Constructor(const Nan::FunctionCallbackInfo<Value> &info)
    {
        // Local<Context> context = info.GetIsolate()->GetCurrentContext();
        if (!info[0]->IsObject())
        {
            return Nan::ThrowTypeError("Object expected");
        }

        if (info.IsConstructCall())
        {
            Local<Object> weakref = info.This();
            Nan::SetInternalFieldPointer(weakref, 0, new WeakRefWrap(weakref, info));
            info.GetReturnValue().Set(weakref);
        }
    }

    void WeakRef::Raw(const Nan::FunctionCallbackInfo<Value> &info)
    {
        WeakRefWrap *wrap = UnWrap(info.This());
        if (wrap && !wrap->target.IsEmpty())
        {
            info.GetReturnValue().Set(Nan::New<Object>(wrap->target));
            return;
        }
        info.GetReturnValue().Set(Nan::Undefined());
    }

    void WeakRefExport(Local<Object> exports)
    {
        Local<Context> context = exports->CreationContext();
        Nan::HandleScope scope;

        // Prepare constructor template
        Local<FunctionTemplate> tpl = Nan::New<FunctionTemplate>(WeakRef::Constructor);
        tpl->SetClassName(Nan::New("WeakRef").ToLocalChecked());
        tpl->InstanceTemplate()->SetInternalFieldCount(1);

        // Prototype
        Nan::SetPrototypeMethod(tpl, "raw", WeakRef::Raw);


        WeakRef::Template.Reset(tpl->GetFunction(context).ToLocalChecked());

        Nan::Set(exports, Nan::New<String>("WeakRef").ToLocalChecked(), Nan::New<Function>(WeakRef::Template));
    }

} // namespace hi_nodejs